
#include "webctrl.h"
#include "webctrlint.h"
#include "events.h"
#include "site.h"
#include "storage.h"
#include "dochost.h"
#include "frame.h"
#include <mshtml.h>

WebCtrlInterface *CreateWebCtrl(WebHostInterface *aWebHost, int aX, int aY, int aWidth, int aHeight)
{
   return new WebCtrlClass(aWebHost, aX, aY, aWidth, aHeight);
}

WebCtrlClass::WebCtrlClass(WebHostInterface *aWebHost, int aX, int aY, int aWidth, int aHeight) : WebHost(aWebHost)
{
   Storage = new NullStorageClass;
   DocHostUiHandler = new DocHostUiHandlerClass(this);
   WebBrowserEvent = new WebBrowserEventClass(this);
   Site = new OleSiteClass(this);
   Frame = new OleFrameClass(this);
   CreateEmbeddedWebControl(aX, aY, aWidth, aHeight);
}

WebCtrlClass::~WebCtrlClass()
{
   UnCreateEmbeddedWebControl();
   delete Storage;
   Storage = 0;
   delete DocHostUiHandler;
   DocHostUiHandler = 0;
   delete WebBrowserEvent;
   WebBrowserEvent = 0;
   delete Site;
   Site = 0;
   delete Frame;
   Frame = 0;
}

void WebCtrlClass::CreateEmbeddedWebControl(int aX, int aY, int aWidth, int aHeight)
{
   OleCreate(CLSID_WebBrowser, IID_IOleObject, OLERENDER_DRAW, 0, Site, Storage, (void **)&WebObject);
   WebObject->SetHostNames(L"Web Host", L"Web View");

   // I have no idea why this is necessary. remark it out and everything works perfectly.
   OleSetContainedObject(WebObject, TRUE);

   RECT rect = {aX, aY, aWidth, aHeight};
   WebObject->DoVerb(OLEIVERB_SHOW, NULL, Site, -1, WebHost->GetHwnd(), &rect);

   IWebBrowser2 *iBrowser;
   WebObject->QueryInterface(IID_IWebBrowser2, (void **)&iBrowser);

   VARIANT vURL;
   vURL.vt = VT_BSTR;
   vURL.bstrVal = SysAllocString(L"about:blank");
   VARIANT ve1, ve2, ve3, ve4;
   ve1.vt = VT_EMPTY;
   ve2.vt = VT_EMPTY;
   ve3.vt = VT_EMPTY;
   ve4.vt = VT_EMPTY;

   iBrowser->put_Left(aX);
   iBrowser->put_Top(aY);
   iBrowser->put_Width(aWidth);
   iBrowser->put_Height(aHeight);

   iBrowser->Navigate2(&vURL, &ve1, &ve2, &ve3, &ve4);

   VariantClear(&vURL);
   iBrowser->Release();
   Connect();
}

void WebCtrlClass::Connect()
{
   if(!WebObject)
   {
      ::MessageBox(0, "BrowserObject is null in Connect", "WebCtrl", MB_OK);
      return;
   }
   IConnectionPointContainer *cpc;
   IConnectionPoint *cp;

   if(WebObject->QueryInterface(IID_IConnectionPointContainer, reinterpret_cast<void **>(&cpc)) == S_OK)
   {
      if(cpc->FindConnectionPoint(DIID_DWebBrowserEvents2, &cp) == S_OK)
      {
         ConnectCookie = 1;
         cp->Advise(static_cast<IDispatch *>(WebBrowserEvent), &ConnectCookie);
      }
   }
}

void WebCtrlClass::UnCreateEmbeddedWebControl(void)
{
   IConnectionPointContainer *cpc;
   IConnectionPoint *cp;

   if(WebObject->QueryInterface(IID_IConnectionPointContainer, reinterpret_cast<void **>(&cpc)) == S_OK)
   {
      if(cpc->FindConnectionPoint(DIID_DWebBrowserEvents2, &cp) == S_OK)
      {
         ConnectCookie = 1;
         cp->Unadvise(ConnectCookie);
      }
   }

   WebObject->Close(OLECLOSE_NOSAVE);
   WebObject->Release();
}

long WebCtrlClass::ShowUrl(const std::string &aUrl)
{
   IWebBrowser2 *webBrowser2;
   VARIANT      myURL;

   if(!WebObject->QueryInterface(IID_IWebBrowser2, (void **)&webBrowser2))
   {
      VariantInit(&myURL);
      myURL.vt = VT_BSTR;
#ifndef UNICODE
      {
         wchar_t *buffer;
         DWORD   size;

         size = MultiByteToWideChar(CP_ACP, 0, aUrl.c_str(), -1, 0, 0);
         if ((buffer = (wchar_t *)GlobalAlloc(GMEM_FIXED, sizeof(wchar_t) *size)) == 0)
         {
            webBrowser2->Release();
            return(-11);
         }
         MultiByteToWideChar(CP_ACP, 0, aUrl.c_str(), -1, buffer, size);
         myURL.bstrVal = SysAllocString(buffer);
         GlobalFree(buffer);
      }
#else
      myURL.bstrVal = SysAllocString(url);
#endif
      if(!myURL.bstrVal)
      {
         webBrowser2->Release();
         return(-6);
      }

      webBrowser2->Navigate2(&myURL, 0, 0, 0, 0);
      VariantClear(&myURL);
      webBrowser2->Release();

      return 0;
   }

   MessageBox(0, "Something went wrong!", "WebCtrl", MB_OK);
   return(-5);
}

long WebCtrlClass::ShowHtml(const std::string &aHtml)
{
   static const SAFEARRAYBOUND ArrayBound = {1, 0};

   IWebBrowser2    *webBrowser2;
   LPDISPATCH       lpDispatch;
   IHTMLDocument2  *htmlDoc2;
   SAFEARRAY       *sfArray;
   VARIANT         *pVar;
   BSTR             bstr = 0;


   if(!WebObject->QueryInterface(IID_IWebBrowser2, (void**)&webBrowser2))
   {
      if(!webBrowser2->get_Document(&lpDispatch))
      {
         if(!lpDispatch->QueryInterface(IID_IHTMLDocument2, (void**)&htmlDoc2))
         {
            /* The HTML passed to IHTMLDocument2::write must be a BSTR within an array of VARIANTs....  */
            if ((sfArray = SafeArrayCreate(VT_VARIANT, 1, (SAFEARRAYBOUND *)&ArrayBound)) != 0)
            {
               if (!SafeArrayAccessData(sfArray, (void**)&pVar))
               {
                  pVar->vt = VT_BSTR;
#ifndef UNICODE
                  {
                     wchar_t  *buffer;
                     DWORD    size;

                     size = MultiByteToWideChar(CP_ACP, 0, aHtml.c_str(), -1, 0, 0);
                     if((buffer = (wchar_t *)GlobalAlloc(GMEM_FIXED, sizeof(wchar_t) * size)) == 0)
                     {
                        htmlDoc2->Release();
                        return -11;
                     }
                     MultiByteToWideChar(CP_ACP, 0, aHtml.c_str(), -1, buffer, size);
                     bstr = SysAllocString(buffer);
                     GlobalFree(buffer);
                  }
#else
                  bstr = SysAllocString(html_txt);
#endif
                  if((pVar->bstrVal = bstr) != 0)
                  {
                     if (htmlDoc2->write(sfArray) != S_OK)
                     {
                         throw ("Could not write sfArray");
                     }

                     /*
                        If the document were not closed, subsequent calls to DisplayHTMLStr would append to the contents of the page.
                     */
                     htmlDoc2->close();
                  }
               }
               /*
                 Freeing the array along with the VARIANT that SafeArrayAccessData created and the BSTR that was allocated with SysAllocString
               */
               SafeArrayDestroy(sfArray);
            }
         }
         lpDispatch->Release();
      }
      webBrowser2->Release();
   }

  if (bstr)
     return 0;

  return(-1);
}


void WebCtrlClass::SetSize(int aWidth, int aHeight)
{
   IWebBrowser2 *iBrowser;
   if(WebObject->QueryInterface(IID_IWebBrowser2, (void **)&iBrowser) == S_OK)
   {
      iBrowser->put_Width(aWidth);
      iBrowser->put_Height(aHeight);
      iBrowser->Release();
   }
}
