// This file implements the webhost test application
// It contains the implementation of WebHostWndClass and WinMain
#include <windows.h>
#include <commctrl.h>
#include <string>
#include "webhost.h"

WebHostWndClass::WebHostWndClass() : WebCtrl(0)
{}

WebHostWndClass::~WebHostWndClass()
{
   delete WebCtrl;
}

bool WebHostWndClass::Create(HINSTANCE hAppInstance)
{
   Instance = hAppInstance;
   char ClassName[] = "WebCtrlClass";
   WNDCLASSEX wc =
   {
      sizeof(wc),
      0,
      WndProcEntry,
      0,
      0,
      hAppInstance,
      LoadIcon(NULL, IDI_APPLICATION),
      LoadCursor(NULL, IDC_ARROW),
      (HBRUSH )(COLOR_BTNFACE + 1),
      NULL,
      ClassName,
      LoadIcon(NULL,IDI_APPLICATION)
   };
   RegisterClassEx(&wc);

   hwnd = CreateWindowEx(0,
                         ClassName,
                         "WebCtrl",
                         WS_OVERLAPPEDWINDOW,
                         CW_USEDEFAULT,0,CW_USEDEFAULT,0,
                         HWND_DESKTOP,
                         NULL,
                         hAppInstance,
                         this);
   if(hwnd)
   {
      ShowWindow(hwnd, SW_SHOW);
   }
   return hwnd != 0;
}

LRESULT CALLBACK WebHostWndClass::WndProcEntry(HWND aHwnd, UINT aMsg, WPARAM wParam, LPARAM lParam)
{
   WebHostWndClass *Me;
   if(aMsg == WM_CREATE)
   {
      CREATESTRUCT *CreateStruct = (CREATESTRUCT *)lParam;
      Me = (WebHostWndClass *)CreateStruct->lpCreateParams;
      if(Me)
      {
         SetWindowLong(aHwnd, GWL_USERDATA, (long )Me);
         Me->hwnd = aHwnd;
      }
   }
   else
   {
      Me = (WebHostWndClass *)GetWindowLong(aHwnd, GWL_USERDATA);
      if(!Me || Me->hwnd != aHwnd)
      {
         return DefWindowProc(aHwnd, aMsg, wParam, lParam);
      }
   }
   return Me->MainWndProc(aHwnd, aMsg, wParam, lParam);
}

LRESULT CALLBACK WebHostWndClass::MainWndProc(HWND aHwnd, UINT aMsg, WPARAM wParam, LPARAM lParam)
{
   switch(aMsg)
   {
   case WM_DESTROY:
      PostQuitMessage(0);
      return TRUE;
   case WM_CREATE:
      OnWmCreate();
      return TRUE;
   case WM_COMMAND:
      if(HIWORD(wParam) == BN_CLICKED)
      {
         switch(LOWORD(wParam))
         {
         case HomeButton:
            if(HomeUrl.size())
               WebCtrl->ShowUrl(HomeUrl);
            break;
         case BackButton:
            if(MruList.size() > 1)
            {
               MruList.pop_back();
               WebCtrl->ShowUrl(MruList.back());
            }
            break;
         case GoButton:
            Go();
            break;
         }
      return TRUE;
   }
   case WM_SIZE:
      SendMessage(StatusBarWindow, aMsg, wParam, lParam);
      RECT SBarRect;
      GetWindowRect(StatusBarWindow, &SBarRect);
      int SBarHeight;
      SBarHeight = SBarRect.bottom - SBarRect.top;
      RECT rect;
      GetClientRect(hwnd, &rect);
      WebCtrl->SetSize(rect.right, rect.bottom - 30 - SBarHeight);
      HWND Ctrl;
      Ctrl = GetDlgItem(hwnd, GoButton);
      MoveWindow(Ctrl, rect.right - 55, 5, 50, 20, TRUE);
      Ctrl = GetDlgItem(hwnd, UrlEdit);
      MoveWindow(Ctrl, 120, 5, rect.right - 180, 20, TRUE);
      return TRUE;
   }
   return DefWindowProc(aHwnd, aMsg, wParam, lParam);
}

void WebHostWndClass::OnWmCreate()
{
   CreateWindow("BUTTON",
                "Home",
                WS_CHILD | WS_VISIBLE | WS_TABSTOP |  BS_PUSHBUTTON,
                5, 5, 50, 20,
                hwnd,
                (HMENU )HomeButton,
                Instance,
                0);
   CreateWindow("BUTTON",
                "Back",
                WS_CHILD | WS_VISIBLE | WS_TABSTOP |  BS_PUSHBUTTON,
                60, 5, 50, 20,
                hwnd,
                (HMENU )BackButton,
                Instance,
                0);
   CreateWindowEx(WS_EX_CLIENTEDGE,
                  "EDIT",
                  "",
                  WS_CHILD | WS_VISIBLE | WS_TABSTOP | WS_GROUP | ES_AUTOHSCROLL,
                  115, 5, 140, 20,
                  hwnd,
                  (HMENU )(UrlEdit),
                  Instance,
                  0);
   HWND HwndEdit;
   HwndEdit = GetDlgItem(hwnd, UrlEdit);
   SetWindowLong(HwndEdit, GWL_USERDATA, (LONG )this);
   OldEditProc = (WNDPROC )GetWindowLong(HwndEdit, GWL_WNDPROC);
   SetWindowLong(HwndEdit, GWL_WNDPROC, (LONG )EditProc);
   CreateWindow("BUTTON",
                "Go",
                WS_CHILD | WS_VISIBLE | WS_TABSTOP |  BS_PUSHBUTTON,
                260, 5, 50, 20,
                hwnd,
                (HMENU )GoButton,
                Instance,
                0);
   StatusBarWindow = CreateStatusWindow(WS_CHILD|WS_VISIBLE|
                                        WS_BORDER|SBARS_SIZEGRIP,
                                        "Ready",
                                        hwnd,
                                        StatusBarId);

   HFONT Font;
   Font = (HFONT )GetStockObject(ANSI_VAR_FONT);
   SendDlgItemMessage(hwnd, HomeButton, WM_SETFONT, (WPARAM )Font, 1);
   SendDlgItemMessage(hwnd, BackButton, WM_SETFONT, (WPARAM )Font, 1);
   SendDlgItemMessage(hwnd, UrlEdit, WM_SETFONT, (WPARAM )Font, 1);
   SendDlgItemMessage(hwnd, GoButton, WM_SETFONT, (WPARAM )Font, 1);
   WebCtrl = CreateWebCtrl(this, 0, 30, 100, 200); // We will set real size at WM_SIZE
}

LRESULT CALLBACK WebHostWndClass::EditProc(HWND HwndEdit, UINT msg, WPARAM wParam, LPARAM lParam)
{
   WebHostWndClass *Parent = (WebHostWndClass *)GetWindowLong(HwndEdit, GWL_USERDATA);
   if(!Parent)
      return 0;
   if(msg == WM_KEYDOWN && wParam == VK_RETURN)
   {
      Parent->Go();
      return 0;
   }
   #ifdef BORLAND_BUILDER
   return CallWindowProc((int CALLBACK (*)())Parent->OldEditProc, HwndEdit, msg, wParam, lParam);
   #else
   return CallWindowProc(Parent->OldEditProc, HwndEdit, msg, wParam, lParam);
   #endif
}

void WebHostWndClass::Go()
{
   std::string Url = GetDlgItemString(hwnd, UrlEdit);
   WebCtrl->ShowUrl(Url);
}

void WebHostWndClass::SetUrl(BSTR aStr)
{
   std::string Text = WideCharToStdString(aStr);
   SetDlgItemText(hwnd, UrlEdit, Text.c_str());

   std::vector<std::string >::iterator it;
   for(it = MruList.begin(); it != MruList.end(); )
   {
      if(*it == Text)
      {
         it = MruList.erase(it);
      }
      else
      {
         ++it;
      }
   }
   MruList.push_back(Text);
}

void WebHostWndClass::SetStatusText(LPCOLESTR aStatusText)
{
   std::string Text = WideCharToStdString(aStatusText);
   SendMessage(StatusBarWindow, SB_SETTEXT, 0, (LPARAM )Text.c_str());
}

int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR lpszCmdLine, int)
{
   OleInitialize(0);
   InitCommonControls();

   WebHostWndClass *WebHostWnd = new WebHostWndClass;

   if(!WebHostWnd->Create(hInstance))
   {
      delete WebHostWnd;
      MessageBox(0, "Failed to create main window", "WebCtrl", MB_OK);
      return 0;
   }
   #if 0 // change this to #if 1 to make it start with a url.
   if(lpszCmdLine && lpszCmdLine[0])
      WebHostWnd->WebCtrl->ShowUrl(lpszCmdLine);
   else
      WebHostWnd->WebCtrl->ShowUrl("http://www.google.com/");
   #else
   std::string Entry =
      "<html><head>"
      "<title>WebHost-Test</title>"
      "</head><body>"
      "<h1>This show how to use the control to show html text</h1>"
      "<ul>"
      "<li><a href='http://www.google.com'>www.google.com</a>"
      "<li><a href='http://home20.inet.tele.dk/midgaard/sample.html'>Midgaard</a>"
      "<li><a href='http://www.yahoo.com'>www.yahoo.com</a>"
      "</ul>"
      "</body></html>";
   WebHostWnd->WebCtrl->ShowHtml(Entry);
   #endif

   WebHostWnd->HomeUrl = "http://home20.inet.tele.dk/midgaard/sample.html";

   MSG msg;
   while(GetMessage(&msg, 0, 0, 0))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   delete WebHostWnd;
   OleUninitialize();
   return msg.wParam;
}
